@@ -1,4 +1,4 @@
-VERSION = '0.2'
+VERSION = '0.2.1'
 VERSION_COPY = f'''\
 Copyright (C) 2023 Marco Bonelli
 Licensed under the GNU General Public License v3.0
@@ -7,3 +7,6 @@ VERSION_HELP = f'''\
 Systrack version {VERSION}
 {VERSION_COPY}\
 '''
+
+if __name__ == '__main__':
+	print(VERSION)
